package external.wha;

import jade.content.AgentAction;
import jade.content.Concept;
import jade.content.ContentElement;
import jade.content.lang.Codec;
import jade.content.lang.Codec.CodecException;
import jade.content.onto.OntologyException;
import jade.content.onto.basic.Action;
import jade.core.AID;
import jade.domain.FIPAAgentManagement.NotUnderstoodException;
import jade.domain.FIPANames.InteractionProtocol;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.proto.SimpleAchieveREResponder;
import misc.AgentHelper;
import ontology.ConfirmationResponse;
import ontology.OrderConfirmation;
import ontology.Reminder;
import external.WholesalerAgent;

/**
 * 
 * Handles responding to reminders from OAs.
 *
 */
public class ReminderBehaviour extends SimpleAchieveREResponder {
	private final WholesalerAgent theAgent;

	public ReminderBehaviour(WholesalerAgent theAgent) {
		super(theAgent, MessageTemplate.and(
				createMessageTemplate(InteractionProtocol.FIPA_REQUEST),
				MessageTemplate.and(MessageTemplate
						.MatchOntology(theAgent.contentOntology.getName()),
						MessageTemplate.MatchLanguage(theAgent.contentLanguage
								.getName()))));
		this.theAgent = theAgent;
	}

	protected ACLMessage prepareResponse(ACLMessage msg) {
		ACLMessage result = msg.createReply();
		ContentElement content = null;
		AgentHelper.debugMessage(theAgent, "got reminder from "
				+ msg.getSender().getLocalName() + ", processing");

		theAgent.addBehaviour(new ReminderBehaviour(theAgent));

		try {
			try {
				content = theAgent.getContentManager().extractContent(msg);
			} catch (OntologyException e) {
				AgentHelper.errorMessage(theAgent,
						"Error relating to message and " + "ontology: " + e);
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			} catch (Codec.CodecException e) {
				AgentHelper.errorMessage(theAgent,
						"Error parsing the message format: " + e);
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			}

			Concept action_content;

			if (content == null || !(content instanceof Action)) {
				AgentHelper.errorMessage(theAgent,
						"Message body was not an action");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			}
			action_content = ((Action) content).getAction();

			if (action_content == null) {
				AgentHelper.errorMessage(theAgent,
						"Message body was not present");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			} else {
				if (action_content instanceof Reminder) {
					// register action
					Reminder information_request = (Reminder) action_content;

					OrderConfirmation reminderConf = information_request
							.getOrderConfirmation();
					ConfirmationResponse resp = new ConfirmationResponse();
					OrderConfirmation newConf = getNewConfirmation(reminderConf);

					if (newConf != null) {
						// re-confirm
						result.setPerformative(ACLMessage.INFORM);
						resp.setOrderConfirmation(newConf);
					} else {
						// no confirmation
						// TODO
						result.setPerformative(ACLMessage.FAILURE);
					}

					AgentAction actioncontent = resp;
					Action action = new Action(new AID(), actioncontent);

					try {
						// fill contents of response
						theAgent.getContentManager().fillContent(result, action);
					} catch (CodecException e) {
						AgentHelper.errorMessage(theAgent,
								"Error relating to message and " + "ontology: "
										+ e);
						throw new NotUnderstoodException("("
								+ result.toString() + ")");
					} catch (OntologyException e) {
						AgentHelper.errorMessage(theAgent,
								"Error parsing the message format: " + e);
						throw new NotUnderstoodException("("
								+ result.toString() + ")");
					}
				} else {
					AgentHelper.errorMessage(theAgent,
							"Message body was not a correct action");
					throw new NotUnderstoodException("(" + result.toString()
							+ ")");
				}
			}
		} catch (NotUnderstoodException e) {
			result.setPerformative(ACLMessage.NOT_UNDERSTOOD);
			result.setContent(e.getMessage());
		}
		return result;
	}

	protected ACLMessage prepareResultNotification(ACLMessage request,
			ACLMessage response) {
		return null;
	}

	private OrderConfirmation getNewConfirmation(OrderConfirmation reminder) {
		// now just returns the same terms
		// if (reminder != null)
		// ;
		return reminder;
	}
}
