package internal;

import internal.oa.OrderingBehaviour;
import jade.content.ContentManager;
import jade.content.lang.sl.SLCodec;
import jade.content.onto.Ontology;
import jade.core.AID;
import jade.core.Agent;
import misc.AgentHelper;
import ontology.LogisticsOntology;

/**
 * 
 * Represents OA in Logistics System.
 * Told by LA to fulfil orders and send reminders, communicates with WhAs.
 *  
 * Dependencies:
 * AgentHelper.extractSuppliers
 * AgentHelper.extractOffer
 * AgentHelper.extractOrder
 * AgentHelper.determineCommand
 * internal.oa package
 *  
 */
public class OrderingAgent extends Agent {

	public static final int RESPONSE_TIMEOUT = 5000; // timeout for WhA responses
	public SLCodec contentLanguage;
	public Ontology contentOntology;
	public AID myLA; // LA of this OA

	// ranking weights go here:
	public double weightPrice = 0.2;
	public double weightTime = 0.2;
	public double weightAmount = 0.2;
	public double weightReliability = 0.2;
	public double weightStrategy = 0.2;

	public double strategyFactor = 0;

	// initialization
	protected void setup() {
		AgentHelper.debugMessage(this, "started");

		contentLanguage = new SLCodec();
		contentOntology = LogisticsOntology.getInstance();
		ContentManager manager = getContentManager();
		manager.registerLanguage(contentLanguage);
		manager.registerOntology(contentOntology);

		// OA knows its LA
		myLA = new AID("la", false); // local AID

		// set up main behaviour
		addBehaviour(new OrderingBehaviour(this));
	}
}