package internal;

import internal.sda.PredictionBehaviour;
import jade.content.ContentManager;
import jade.content.lang.sl.SLCodec;
import jade.content.onto.Ontology;
import jade.core.AID;
import jade.core.Agent;
import misc.AgentHelper;
import ontology.LogisticsOntology;
import ontology.PredictionDescription;
import ontology.Product;

/**
 * 
 * Represents SDA in Logistics System.
 * Makes predictions regarding demand of goods, which it forwards to WA.
 *
 */
public class SDAgent extends Agent {
	public SLCodec contentLanguage;
	public Ontology contentOntology;
	public AID myWA; // WA of the shop this SDA belongs to

	// initialization
	protected void setup() {
		AgentHelper.debugMessage(this, "started");

		contentLanguage = new SLCodec();
		contentOntology = LogisticsOntology.getInstance();

		ContentManager manager = getContentManager();
		manager.registerLanguage(contentLanguage);
		manager.registerOntology(contentOntology);

		// our WA is known at startup: 
		myWA = new AID("wa", false); // this is local name, if global, set true
		// *****

		// give time for all agents to start and register with CICAgent
		doWait(1000);

		// to invoke prediction for testing
		PredictionDescription desc = createPrediction();
		addBehaviour(new PredictionBehaviour(this, desc));
		// **
	}

	// creates a generic random prediction for someproduct
	private PredictionDescription createPrediction() {
		// "predict" here
		String productName = "someproduct";
		float amount = (float) Math.random();
		float dev = (float) Math.random();
		int period = (int) Math.round(Math.random() * 100);
		
		// initialize PredictionDescription
		PredictionDescription desc = new PredictionDescription();
		Product p = new Product();
		p.setProductName(productName);
		desc.setTheProduct(p);
		desc.setPredictionAmount(amount);
		desc.setPredictionDeviation(dev);
		desc.setPredictionPeriod(period);

		return desc;
	}
}