package internal;

import internal.wa.MonitoringBehaviour;
import internal.wa.PredictionBehaviour;
import jade.content.ContentManager;
import jade.content.lang.sl.SLCodec;
import jade.content.onto.Ontology;
import jade.core.AID;
import jade.core.Agent;

import java.util.ArrayList;

import misc.AgentHelper;
import ontology.LogisticsOntology;
import ontology.PredictionDescription;

/**
 * 
 * Represents Warehouse Agent in scope of Logistics system. WA monitors stock,
 * sends delivery notifications to LA, receives predictions from SDA.
 * 
 * Dependencies: AgentHelper.extractPredictionDescription
 * 
 */
public class WarehouseAgent extends Agent {
	public SLCodec contentLanguage;
	public Ontology contentOntology;

	public final int MONITORING_INTERVAL = 10000; // time between every stock
													// check (in ms)

	public ArrayList predictionList; // contains each prediction received
										// from SDA that is monitored
										// (<PredictionDescription>)
	public ArrayList lastCheckTime; // contains timestamps for last check of
										// each prediction (<Integer>)
	public AID myLA; // adress of LA
	private int time = 0; // timer

	// private boolean ordered = false; // temp for testing

	// initialization
	protected void setup() {
		AgentHelper.debugMessage(this, "started");

		predictionList = new ArrayList();
		lastCheckTime = new ArrayList();

		contentLanguage = new SLCodec();
		contentOntology = LogisticsOntology.getInstance();

		ContentManager manager = getContentManager();
		manager.registerLanguage(contentLanguage);
		manager.registerOntology(contentOntology);

		// our LA is known at startup:
		myLA = new AID("la", false); // this is local name, if global, set
										// true

		addBehaviour(new PredictionBehaviour(this));
		addBehaviour(new MonitoringBehaviour(this));
	}

	// adds new prediction from SDA
	public void modifyPredictions(PredictionDescription desc) {
		predictionList.add(desc);
		lastCheckTime.add(new Integer(Integer.MAX_VALUE));
	}

	// clock tick
	public void increaseTime() {
		time++;
	}

	// gets current 'time'
	public int getTime() {
		return time;
	}

	// returns current stock level for product, always returns 0
	// (out of scope - should be implemented as part of e-store)
	public int getStockForProduct() {
		return 0;
	}
}