package internal.cic;

import internal.CICAgent;
import jade.content.Concept;
import jade.content.ContentElement;
import jade.content.lang.Codec;
import jade.content.onto.OntologyException;
import jade.content.onto.basic.Action;
import jade.domain.FIPAAgentManagement.NotUnderstoodException;
import jade.domain.FIPAAgentManagement.RefuseException;
import jade.domain.FIPANames.InteractionProtocol;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.proto.SimpleAchieveREResponder;
import misc.AgentHelper;
import ontology.CICDeregister;
import ontology.CICRegister;

// handles WhA registrations/deregistrations
public class RegistrationBehaviour extends SimpleAchieveREResponder {
	private final CICAgent theAgent;

	public RegistrationBehaviour(CICAgent agent) {
		super(
				agent,
				MessageTemplate.and(
						createMessageTemplate(InteractionProtocol.FIPA_REQUEST),
						MessageTemplate.and(
								MessageTemplate.MatchOntology(agent.contentOntology.getName()),
								MessageTemplate.MatchLanguage(agent.contentLanguage.getName()))));
		this.theAgent = agent;
	}

	protected ACLMessage prepareResponse(ACLMessage msg)
			throws NotUnderstoodException, RefuseException {
		return null;
	}

	// process request and answer
	protected ACLMessage prepareResultNotification(ACLMessage request,
			ACLMessage resp) {
		ACLMessage response = request.createReply();
		ContentElement content = null;
		AgentHelper.debugMessage(theAgent, "got register request from "
				+ request.getSender().getLocalName() + ", processing");

		// allow CIC to serve more requests while processing this one
		theAgent.addBehaviour(new RegistrationBehaviour(theAgent));

		try {
			try {
				content = theAgent.getContentManager().extractContent(
						request);
			} catch (OntologyException e) {
				AgentHelper.errorMessage(theAgent, "Error relating to message and "
						+ "ontology: " + e);
				throw new NotUnderstoodException("(" + request.toString() + ")");
			} catch (Codec.CodecException e) {
				AgentHelper.errorMessage(theAgent, "Error parsing the message format: " + e);
				throw new NotUnderstoodException("(" + request.toString() + ")");
			}

			Concept action_content;

			if (content == null || !(content instanceof Action)) {
				AgentHelper.errorMessage(theAgent, "Message body was not an action");
				throw new NotUnderstoodException("(" + request.toString() + ")");
			}
			action_content = ((Action) content).getAction();

			if (action_content == null) {
				AgentHelper.errorMessage(theAgent, "Message body was not present");
				throw new NotUnderstoodException("(" + request.toString() + ")");
			} else {
				if (action_content instanceof CICRegister) {
					// we got register request
					CICRegister information_request = (CICRegister) action_content;

					if (information_request.getAgentCICDescription()
							.getAgentAID()
							.compareTo(request.getSender()) != 0) {
						response.setPerformative(ACLMessage.REFUSE);
						return response;
					}
					theAgent.doRegistration(((CICRegister) action_content).getAgentCICDescription(), true);
				} else if (action_content instanceof CICDeregister) {
					// deregister request
					CICDeregister information_request = (CICDeregister) action_content;

					if (information_request.getAgentAID().compareTo(
							request.getSender()) != 0) {
						response.setPerformative(ACLMessage.REFUSE);
						return response;
					}
					theAgent.doRegistration(((CICRegister) action_content).getAgentCICDescription(), false);
				} else {
					AgentHelper.errorMessage(theAgent, "Message body was not a correct action");
					throw new NotUnderstoodException("(" + request.toString()
							+ ")");
				}
			}
		} catch (NotUnderstoodException e) {
			response.setPerformative(ACLMessage.NOT_UNDERSTOOD);
			response.setContent(e.getMessage());
		}

		response.setPerformative(ACLMessage.INFORM);
		return response;
	}
}