package internal.la;

import internal.LogisticsAgent;
import jade.content.AgentAction;
import jade.content.lang.Codec.CodecException;
import jade.content.onto.OntologyException;
import jade.content.onto.basic.Action;
import jade.domain.FIPANames.InteractionProtocol;
import jade.lang.acl.ACLMessage;
import jade.proto.SimpleAchieveREInitiator;
import jade.util.leap.ArrayList;
import misc.AgentHelper;
import ontology.CICQuery;
import ontology.OrderDescription;
import ontology.Product;

// handles getting the supplier list from CIC
public class ListBehaviour extends SimpleAchieveREInitiator {
	private ArrayList theList;

	private boolean success = false;

	private IssueOrderBehaviour theMaster;
	private final LogisticsAgent theAgent;

	ListBehaviour(IssueOrderBehaviour master, ACLMessage request) {
		super(master.theAgent, request);
		theMaster = master;
		theList = theMaster.theSuppliers;
		theAgent = theMaster.theAgent;
	}

	// prepare request message
	protected ACLMessage prepareRequest(ACLMessage msg) {
		OrderDescription order = theMaster.theOrder;
		if (order != null) {
			// we got order, so prepare query based on product
			theMaster.cicQueryMessage = prepareCICQuery(theAgent, order
					.getTheProduct());
			return theMaster.cicQueryMessage;
		} else
			// order is empty (FSM always enters this state, error or not)
			// so no message - behaviour will end without sending anything
			return null;
	}

	public void reset(ACLMessage msg) {
		super.reset(msg);
	}

	protected void handleAgree(ACLMessage msg) {
		AgentHelper.debugMessage(theAgent, msg.getSender().getLocalName()
				+ " agreed to send list");
	}

	protected void handleRefuse(ACLMessage msg) {
		AgentHelper.debugMessage(theAgent, msg.getSender().getLocalName()
				+ " refused to send list");
	}

	// CIC responded
	protected void handleInform(ACLMessage msg) {
		AgentHelper.debugMessage(theAgent, msg.getSender().getLocalName()
				+ " sent me the list");

		// initialize supplier list from the message
		theMaster.setSupplierList(theAgent, msg, theList);

		if (theList.size() > 0) {
			// we got at least one supplier, we can try ordering
			success = true;
		}
	}

	protected void handleNotUnderstood(ACLMessage msg) {
		AgentHelper.debugMessage(theAgent, msg.getSender().getLocalName()
				+ " did not understand my request:" + msg.getContent());
	}

	protected void handleOutOfSequence(ACLMessage msg) {
		AgentHelper.debugMessage(theAgent, msg.getSender().getLocalName()
				+ " sent me unexpected message:" + msg.getContent());
	}

	// FSM result
	public int onEnd() {
		if (success) {
			// we got list with some suppliers
			AgentHelper.debugMessage(theAgent, "supplier list fetch success");
			// prepare template message for OA
			theMaster.issueOrderTemplate = theMaster.prepareIssueOrder(theMaster);
			return 0;
		} else
			// we didnt get anything useful from CIC
			AgentHelper.debugMessage(theAgent, "supplier list fetch failed");
		return 1;
	}

	// prepare query to CIC
	private ACLMessage prepareCICQuery(LogisticsAgent theAgent, Product product) {
		ACLMessage request = new ACLMessage(ACLMessage.QUERY_REF);
		request.setSender(theAgent.getAID());
		request.setLanguage(theAgent.contentLanguage.getName());
		request.setOntology(theAgent.contentOntology.getName());

		request.addReceiver(theAgent.myCIC);
		request.setProtocol(InteractionProtocol.FIPA_QUERY);

		CICQuery query = new CICQuery();
		// set the product we are interested in
		query.setTheProduct(product);

		AgentAction content = query;
		Action action = new Action(theAgent.myCIC, content);

		try {
			// fill contents of query
			theAgent.getContentManager().fillContent(request, action);
		} catch (CodecException e) {
			AgentHelper.errorMessage(theAgent, "Error relating to message and"
					+ "ontology");
			return null;
		} catch (OntologyException e) {
			AgentHelper.errorMessage(theAgent,
					"Error parsing the message format");
			return null;
		}
		return request;
	}
}
