package internal.oa;

import internal.OrderingAgent;
import ontology.CFPRequest;
import ontology.IssueOrder;
import ontology.OrderDescription;
import ontology.Reminder;
import jade.content.AgentAction;
import jade.content.Concept;
import jade.content.ContentElement;
import jade.content.lang.Codec;
import jade.content.lang.Codec.CodecException;
import jade.content.onto.OntologyException;
import jade.content.onto.basic.Action;
import jade.core.AID;
import jade.core.Agent;
import jade.core.behaviours.OneShotBehaviour;
import jade.domain.FIPAAgentManagement.NotUnderstoodException;
import jade.domain.FIPANames.InteractionProtocol;
import jade.lang.acl.ACLMessage;
import misc.AgentHelper;

// handles receiving messages from LA, deciphering them and deciding what to do
public class DispatchBehaviour extends OneShotBehaviour {
	OrderingBehaviour theMaster;
	private final OrderingAgent theAgent;
	int laCommand; // 0 - order, 1 - reminder

	public DispatchBehaviour(OrderingBehaviour master) {
		theMaster = master;
		theAgent = theMaster.theAgent;
	}

	public void action() {
		// get the message that was received
		theMaster.laCommandMessage = theMaster.retrieveData("lacommand");

		AgentHelper.debugMessage(theAgent,
				"got some message to dispatch");

		// determine the type of command
		laCommand = determineCommand(theAgent, theMaster.laCommandMessage);
		if (laCommand == 0) {
			// we got order - prepare for that
			theMaster.theOrder = OAHelper.extractOrder(theAgent, theMaster.laCommandMessage);
			theMaster.theSuppliers = OAHelper.extractSuppliers(theAgent, theMaster.laCommandMessage);
			theMaster.cfpTemplate = createCFP(theMaster.theOrder);
		} else if (laCommand == 1) {
			// we got reminder - no additional preparations
			;
		} else {
			// unknown command type
			AgentHelper.errorMessage(theAgent, "Illegal message to OA");
		}
	}

	// result for FSM
	public int onEnd() {
		return laCommand;
	}
	
	// create CFP template based on order description
	private ACLMessage createCFP(OrderDescription theOrder) {
		ACLMessage request = new ACLMessage(ACLMessage.CFP);
		request.setSender(theAgent.getAID());
		request.setLanguage(theAgent.contentLanguage.getName());
		request.setOntology(theAgent.contentOntology.getName());
		request.setProtocol(InteractionProtocol.FIPA_CONTRACT_NET);

		CFPRequest cfp = new CFPRequest();
		cfp.setOrderDescription(theOrder);

		AgentAction content = cfp;
		Action action = new Action(new AID(), content);

		try {
			// fill contents of cfp
			theAgent.getContentManager().fillContent(request, action);
		} catch (CodecException e) {
			AgentHelper.errorMessage(theAgent, ": Error creating CFP");
		} catch (OntologyException e) {
			AgentHelper.errorMessage(theAgent, ": Error creating CFP");
		}
		return request;
	}
	
	// determine LA command from LA message 
	private int determineCommand(Agent theAgent, ACLMessage msg) {
		ContentElement content = null;
		int command = -1;

		try {
			try {
				content = theAgent.getContentManager().extractContent(msg);
			} catch (OntologyException e) {
				AgentHelper.errorMessage(theAgent, "Error relating to message and "
						+ "ontology");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			} catch (Codec.CodecException e) {
				AgentHelper.errorMessage(theAgent,
						"Error Parsing the message format");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			}

			Concept action_content;

			if (content == null || !(content instanceof Action)) {
				AgentHelper.errorMessage(theAgent, "Message body was not an action");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			}

			action_content = ((Action) content).getAction();

			if (action_content == null) {
				AgentHelper.errorMessage(theAgent, "Message body was not present");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			} else {
				if (action_content instanceof IssueOrder) {
					command = 0;
				} else if (action_content instanceof Reminder) {
					command = 1;
				} else {
					AgentHelper.errorMessage(theAgent,
							"Message body was not a correct action");
					throw new NotUnderstoodException("(" + msg.toString() + ")");
				}
			}
		} catch (NotUnderstoodException e) {
			;
		}
		return command;
	}
}