package internal.oa;

import internal.OrderingAgent;
import ontology.ConfirmationResponse;
import ontology.OrderConfirmation;
import jade.content.AgentAction;
import jade.content.lang.Codec.CodecException;
import jade.content.onto.OntologyException;
import jade.content.onto.basic.Action;
import jade.core.behaviours.OneShotBehaviour;
import jade.lang.acl.ACLMessage;
import misc.AgentHelper;

// Final state of the FSM
public class FinalStateBehaviour extends OneShotBehaviour {
	OrderingBehaviour theMaster;
	private final OrderingAgent theAgent;

	public FinalStateBehaviour(OrderingBehaviour master) {
		theMaster = master;
		theAgent = theMaster.theAgent;
	}

	public void action() {
		// examine results
		if (theMaster.theConfirmation != null) {
			// order success
			AgentHelper.debugMessage(this.myAgent, "successful");
		} else {
			// order failed
			AgentHelper.debugMessage(this.myAgent, "failed");
		}

		// notify LA
		ACLMessage issueOrderResult = prepareIssueOrderResult(
				theMaster.laCommandMessage, theMaster.theConfirmation);
		myAgent.send(issueOrderResult);

		// this behaviour closes, so set up new one to handle next LA request
		myAgent.addBehaviour(new OrderingBehaviour(myAgent));
	}
	
	// prepare response for LA
	private ACLMessage prepareIssueOrderResult(ACLMessage originalMessage,
			OrderConfirmation theConfirmation) {
		ACLMessage message = originalMessage.createReply();

		ConfirmationResponse conf = new ConfirmationResponse();
		if (theConfirmation != null) {
			// we got confirmation, success
			conf.setOrderConfirmation(theConfirmation);
			message.setPerformative(ACLMessage.INFORM);
		} else {
			// or failure
			message.setPerformative(ACLMessage.FAILURE);
		}

		AgentAction content = conf;
		Action action = new Action(theAgent.myLA, content);

		try {
			// fill contents of inform
			theAgent.getContentManager().fillContent(message, action);
		} catch (CodecException e) {
			AgentHelper.errorMessage(theAgent, "Error relating to message and"
					+ "ontology");
			return null;
		} catch (OntologyException e) {
			AgentHelper.errorMessage(theAgent, "Error parsing the message format");
			e.printStackTrace();
			return null;
		}
		return message;
	}
}
