package internal.oa;

import internal.OrderingAgent;
import jade.core.behaviours.SimpleBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import misc.AgentHelper;
import ontology.OfferDescription;

// handles receiving of proposals from WhAs that were sent cfps
public class ProposalBehaviour extends SimpleBehaviour {

	private final OrderingBehaviour theMaster;
	private int result = 1;
	private final OrderingAgent theAgent;

	public ProposalBehaviour(OrderingBehaviour master) {
		this.theMaster = master;
		theAgent = theMaster.theAgent;
	}

	public void action() {
		// set proposal template
		MessageTemplate mt = MessageTemplate.and(
				MessageTemplate.MatchPerformative(ACLMessage.PROPOSE),
				MessageTemplate.and(
						MessageTemplate.MatchOntology(theAgent.contentOntology.getName()),
						MessageTemplate.MatchLanguage(theAgent.contentLanguage.getName())));
		ACLMessage proposal = null;
		// for all potential responders...
		for (int i = 0; i < theMaster.theSuppliers.size(); i++) {
			// ...wait for message, or until timeout
			proposal = theAgent.blockingReceive(mt, OrderingAgent.RESPONSE_TIMEOUT);
			if (proposal != null) {
				// if proposal received (not timed out) add the proposal to the list
				OfferDescription offer = OAHelper.extractOffer(myAgent, proposal);
				theMaster.addOffer(offer, proposal, theMaster);
			}
		}
		AgentHelper.debugMessage(myAgent, "received "
				+ theMaster.offerList.size() + " offers");
		if (theMaster.offerList.size() > 0)
			// we got some responses
			result = 0;
		else
			// or not
			result = 1;
	}

	// we are done
	public boolean done() {
		return true;
	}

	// result for FSM
	public int onEnd() {
		return result;
	}
}
