package internal.wa;

import internal.WarehouseAgent;
import jade.content.AgentAction;
import jade.content.Concept;
import jade.content.ContentElement;
import jade.content.lang.Codec;
import jade.content.onto.OntologyException;
import jade.content.onto.basic.Action;
import jade.core.behaviours.OneShotBehaviour;
import jade.domain.FIPAAgentManagement.NotUnderstoodException;
import jade.lang.acl.ACLMessage;
import misc.AgentHelper;
import ontology.DeliveryDescription;
import ontology.InformResult;
import ontology.OrderRequestResult;
import ontology.WADelivery;

/**
 * 
 * Sends out delivery notification with an appropriate orderId
 * 
 */
public class DeliveryBehaviour extends OneShotBehaviour {

	private final WarehouseAgent theAgent;

	private ACLMessage theMessage;

	public DeliveryBehaviour(WarehouseAgent agent, ACLMessage msg) {
		super(agent);
		theMessage = msg;
		this.theAgent = agent;
	}

	// just prepare and send the message
	public void action() {
		AgentHelper.debugMessage(theAgent, "sending delivery notification");
		ACLMessage notification = createDeliveryFromConf(theMessage);
		theAgent.send(notification);
	}

	// extracts the orderId we need to confirm delivery from order
	// confirmation
	private ACLMessage createDeliveryFromConf(ACLMessage confMsg) {
		ACLMessage delMsg = confMsg.createReply();
		delMsg.setPerformative(ACLMessage.INFORM);

		ContentElement content = null;

		try {
			try {
				AgentHelper.debugMessage(theAgent, "reading order result");
				content = theAgent.getContentManager().extractContent(confMsg);
			} catch (OntologyException e) {
				AgentHelper.errorMessage(theAgent,
						"Error relating to message and " + "ontology");
				throw new NotUnderstoodException("(" + confMsg.toString()
						+ ")");
			} catch (Codec.CodecException e) {
				AgentHelper.errorMessage(theAgent,
						"Error Parsing the message format");
				throw new NotUnderstoodException("(" + confMsg.toString()
						+ ")");
			}

			Concept action_content;

			if (content == null || !(content instanceof Action)) {
				AgentHelper.errorMessage(theAgent,
						"Message body was not an action");
				throw new NotUnderstoodException("(" + confMsg.toString()
						+ ")");
			}

			action_content = ((Action) content).getAction();

			if (action_content == null) {
				AgentHelper.errorMessage(theAgent,
						"Message body was not present");
				throw new NotUnderstoodException("(" + confMsg.toString()
						+ ")");
			} else {
				if (action_content instanceof InformResult) {
					InformResult informResult = (InformResult) action_content;

					// prepare content for reply based on received data
					String orderId = ((OrderRequestResult) informResult
							.getResult()).getOrderConfirmation()
							.getOrderId();

					DeliveryDescription desc = new DeliveryDescription();
					desc.setOrderId(orderId);
					delMsg.setConversationId(AgentHelper.createConvId(
							theAgent, "delivery-" + orderId));
					WADelivery delivery = new WADelivery();
					delivery.setDeliveryDescription(desc);

					AgentAction actioncontent = delivery;
					Action action = new Action(theAgent.myLA, actioncontent);

					try {
						AgentHelper.debugMessage(theAgent,
								"preparing delivery notification");
						theAgent.getContentManager().fillContent(delMsg, action);
					} catch (OntologyException e) {
						AgentHelper.errorMessage(theAgent,
								"Error relating to message and ontology");
					} catch (Codec.CodecException e) {
						AgentHelper.errorMessage(theAgent,
								"Error Parsing the message format");
					}
				} else {
					AgentHelper.errorMessage(theAgent,
							"Message body was not a correct action");
					throw new NotUnderstoodException("("
							+ confMsg.toString() + ")");
				}
			}
		} catch (NotUnderstoodException e) {
			;
		}
		return delMsg;
	}
}