package internal.wa;

import internal.WarehouseAgent;
import jade.lang.acl.ACLMessage;
import jade.proto.SimpleAchieveREInitiator;
import misc.AgentHelper;

/**
 * 
 * Behaviour handling the ordering requests to LA
 * 
 */
public class OrderRequestBehaviour extends SimpleAchieveREInitiator {

	private final WarehouseAgent theAgent;

	OrderRequestBehaviour(WarehouseAgent agent, ACLMessage request) {
		super(agent, request);
		this.theAgent = agent;
	}

	public void reset(ACLMessage msg) {
		super.reset(msg);
	}

	// agree message sent, that means the order is successful...
	// ... and we can send delivery confirmation message
	// (monitoring deliveries and resulting stock changes is out of scope)
	protected void handleAgree(ACLMessage msg) {
		AgentHelper.debugMessage(theAgent, msg.getSender().getLocalName()
				+ " agreed to order.");
		theAgent.addBehaviour(new DeliveryBehaviour(theAgent, msg));
	}

	protected void handleRefuse(ACLMessage msg) {
		AgentHelper.debugMessage(theAgent, msg.getSender().getLocalName()
				+ " refused to order.");
	}

	// ordering failed :(
	protected void handleFailure(ACLMessage msg) {
		AgentHelper.debugMessage(theAgent, msg.getSender().getLocalName()
				+ " failed to order.");
	}

	protected void handleNotUnderstood(ACLMessage msg) {
		AgentHelper.debugMessage(theAgent, msg.getSender().getLocalName()
				+ " did not understand my request.");
	}

	protected void handleOutOfSequence(ACLMessage msg) {
		AgentHelper.debugMessage(theAgent, msg.getSender().getLocalName()
				+ " sent me unexpected message.");
	}

	// full success
	protected void handleInform(ACLMessage msg) {
		AgentHelper.debugMessage(theAgent, msg.getSender().getLocalName()
				+ " ordered successfully, delivery complete.");
	}
}
