package internal.wa;

import internal.WarehouseAgent;
import jade.content.Concept;
import jade.content.ContentElement;
import jade.content.lang.Codec;
import jade.content.onto.OntologyException;
import jade.content.onto.basic.Action;
import jade.core.Agent;
import jade.core.behaviours.CyclicBehaviour;
import jade.domain.FIPAAgentManagement.NotUnderstoodException;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import misc.AgentHelper;
import ontology.PredictionDescription;
import ontology.SDAPrediction;

/**
 * 
 * Behaviour handling incoming predictions from SDA
 * 
 */
public class PredictionBehaviour extends CyclicBehaviour {
	
	private WarehouseAgent theAgent;
	
	public PredictionBehaviour(WarehouseAgent agent) {
		super(agent);
		theAgent = agent;
	}

	public void onStart() {
		AgentHelper.debugMessage(theAgent, "waiting for predictions");
	}

	public void action() {
		// setup message matching
		MessageTemplate m1 = MessageTemplate
				.MatchPerformative(ACLMessage.REQUEST);
		MessageTemplate m2 = MessageTemplate.MatchLanguage(theAgent.contentLanguage
				.getName());
		MessageTemplate m3 = MessageTemplate.MatchOntology(theAgent.contentOntology
				.getName());
		MessageTemplate m1andm2 = MessageTemplate.and(m1, m2);
		MessageTemplate m1andm2andm3 = MessageTemplate.and(m1andm2, m3);
		// wait for the message
		ACLMessage msg = theAgent.receive(m1andm2andm3);
		if (msg != null) {
			AgentHelper.debugMessage(myAgent, "received prediction");
			// extract description and add it to the list
			PredictionDescription desc = extractPredictionDescription(myAgent, msg);
			theAgent.modifyPredictions(desc);
		} else
			// keep waiting
			block();
	}
	
	// extracts prediction from ACLMessage from SDA
	public static PredictionDescription extractPredictionDescription(Agent theAgent, ACLMessage msg) {
		ContentElement content = null;

		try {
			try {
				content = theAgent.getContentManager().extractContent(msg);
			} catch (OntologyException e) {
				AgentHelper.errorMessage(theAgent, "Error relating to message and "
						+ "ontology");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			} catch (Codec.CodecException e) {
				AgentHelper.errorMessage(theAgent,
						"Error Parsing the message format");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			}

			Concept action_content;

			if (content == null || !(content instanceof Action)) {
				AgentHelper.errorMessage(theAgent, "Message body was not an action");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			}

			action_content = ((Action) content).getAction();

			if (action_content == null) {
				AgentHelper.errorMessage(theAgent, "Message body was not present");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			} else {
				if (action_content instanceof SDAPrediction) {

					SDAPrediction information_request = (SDAPrediction) action_content;

					// get prediction details...
					PredictionDescription thePrediction = information_request.getPredictionDescription();
					// ... and return them
					return thePrediction;
				} else {
					AgentHelper.errorMessage(theAgent,
							"Message body was not a correct action");
					throw new NotUnderstoodException("(" + msg.toString() + ")");
				}
			}
		} catch (NotUnderstoodException e) {
			;
		}
		return null;
	}
}
